<?php

namespace QUI\FRS;

use QUI;
use QUI\Smarty\Collector;

use function defined;
use function md5;
use function php_sapi_name;
use function rand;

class EventHandler
{
    public static function isActive(): bool
    {
        $active = FrontendRequestSigning::getConfig()->getValue('general_frs', 'active');

        if (empty($active)) {
            return false;
        }

        $backendActive = FrontendRequestSigning::getConfig()->getValue('general_frs', 'backend_active');

        if (QUI::isBackend() && !$backendActive) {
            return false;
        }

        return true;
    }

    public static function onTemplateBegin(
        Collector $Collection,
        QUI\Template $Template
    ): void {
        if (!self::isActive()) {
            return;
        }

        $token = md5((string)rand(100000, 999999));

        try {
            $token = FrontendRequestSigning::createToke();
        } catch (\Exception $Exception) {
            QUI\System\Log::addError($Exception->getMessage(), [
                'type' => 'jwt token creation'
            ]);
        }

        $Collection->append(
            '
            <meta content="' . $token . '" name="jwt-token" data-no-cache="1" />
            <script src="' . URL_OPT_DIR . 'quiqqer/frontend-request-signing/bin/frs.js"></script>
            '
        );
    }

    /**
     * @param mixed[] &$result
     * @return void
     */
    public static function onAjaxResult(array &$result): void
    {
        if (!self::isActive()) {
            return;
        }

        $token = md5((string)rand(100000, 999999));

        try {
            $token = FrontendRequestSigning::createToke();
        } catch (\Exception $Exception) {
            QUI\System\Log::addError($Exception->getMessage(), [
                'type' => 'jwt token creation'
            ]);
        }

        $result['Jwt'] = $token;
    }

    public static function onAdminLoadBegin(): void
    {
        if (!self::isActive()) {
            return;
        }

        $token = md5((string)rand(100000, 999999));

        try {
            $token = FrontendRequestSigning::createToke();
        } catch (\Exception $Exception) {
            QUI\System\Log::addError($Exception->getMessage(), [
                'type' => 'csrf token creation'
            ]);
        }

        echo '
            <meta content="' . $token . '" name="jwt-token" data-no-cache="1" />
            <script src="' . URL_OPT_DIR . 'quiqqer/frontend-request-signing/bin/frs.js"></script>
        ';
    }

    public static function onHeaderLoaded(): void
    {
        if (!self::isActive()) {
            return;
        }

        if (php_sapi_name() === 'cli') {
            return;
        }

        if (defined('QUIQQER_CONSOLE') && QUIQQER_CONSOLE) {
            return;
        }

        if (defined('QUIQQER_AJAX')) {
            $headers = getallheaders();

            if (!isset($headers['X-Jwt-Token'])) {
                $headers['X-Jwt-Token'] = '';
            }

            FrontendRequestSigning::validate($headers['X-Jwt-Token']);
        }
    }
}
