<?php

namespace QUI\FRS;

use Firebase\JWT\JWT;
use Firebase\JWT\Key;
use QUI;
use QUI\FRS\Exceptions\InvalidJwtToken;

use function is_string;
use function time;

class FrontendRequestSigning
{
    public static function isValid(string $token): bool
    {
        try {
            self::validate($token);
            return true;
        } catch (QUI\Exception) {
        }

        return false;
    }

    /**
     * @param $token
     * @return void
     *
     * @throws InvalidJwtToken|QUI\ExceptionStack
     */
    public static function validate(string $token): void
    {
        $User = QUI::getUserBySession();
        $key = self::getKey();

        try {
            $decoded = JWT::decode($token, new Key($key, 'HS256'));
        } catch (\Exception) {
            throw new InvalidJwtToken();
        }

        $userId = $decoded->user ?? null;
        $started = $decoded->iat ?? null;
        $expired = $decoded->exp ?? null;
        $lifetime = 3600;

        $host = $decoded->iss ?? null;
        $path = $decoded->path ?? null;

        if ($userId !== $User->getUUID()) {
            throw new InvalidJwtToken();
        }

        if ($started + $lifetime < time()) {
            throw new InvalidJwtToken();
        }

        if ($expired < time()) {
            throw new InvalidJwtToken();
        }


        QUI::getEvents()->fireEvent('onFrontendRequestSigningValidate', [$decoded]);
    }

    public static function createToke(): string
    {
        $User = QUI::getUserBySession();
        $key = self::getKey();

        $payload = [
            'iss' => $_SERVER['HTTP_HOST'],
            'path' => $_SERVER['REQUEST_URI'],
            'user' => $User->getUUID(),
            'iat' => time(),
            'exp' => time() + 3600
        ];

        QUI::getEvents()->fireEvent('onFrontendRequestSigningCreateToken', [&$payload]);

        /*
        $Project = QUI::getRewrite()->getProject();
        $Site = QUI::getRewrite()->getSite();

        'project' => $Project->getName(),
        'lang' => $Project->getLang(),
        'site' => $Site->getId(),
        */

        return JWT::encode($payload, $key, 'HS256');
    }

    protected static function getKey(): string
    {
        $Config = self::getConfig();
        $key = $Config->getValue('general_frs', 'key');

        if (empty($key)) {
            $key = QUI\Utils\Security\Orthos::getPassword(15);

            $Config->setValue('general_frs', 'key', $key);
            $Config->save();
        }

        if (is_string($key)) {
            return $key;
        }

        return '';
    }

    public static function getConfig(): QUI\Config
    {
        $Config = QUI::getPackage('quiqqer/frontend-request-signing')->getConfig();

        if ($Config) {
            return $Config;
        }

        throw new QUI\Exception('Frontend Request Signing Config not found');
    }
}
